/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Volume management
*  SCOPE :              Implements volumes for YOODA
*  FILE :               volume.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __volume_HXX
#define __volume_HXX

/* IMPORTS *******************************************************************/

#ifndef __fnctl_H
#define __fnctl_H
#include <fcntl.h>
#endif

#ifndef __unistd_H
#define __unistd_H
#include <unistd.h>
#endif

#ifndef __class_HXX
#include "class.hxx"
#endif

#ifndef __oid_HXX
#include "oid.hxx"
#endif

#ifndef __yoofunc_HXX
#include "yoofunc.hxx"
#endif

#include "alphadecl.h"

/*****************************************************************************/

class Volume {

  private :

  int _fd ;               // File descriptor of the volume
  int _fdl ;              // File descriptor of the volume lock file
  char* _volname ;        // Complete name of the volume
  int _volid ;            // Identifier of the volume inthe YOODA DB
  int _size ;             // Current number of pages
  VolAllocator* _allocator ; // Page allocator for the volume

  public :
  
  Volume(char* name, int id) ; // build or open a volume on the file 'name'
  ~Volume() ;                  // Close the volume
  int volid() ;
  int read(Page* p, PID pid) ; // Read page design by pid into p  
  int write(Page* p, PID pid); // Write page design by pid from p
  PID alloc(int number) ;      // Allocation of pages  
  void sync() ;                 // Synchronise the volume on the disk
  void open() ;
  void freepage(PID& pid) ;    // set free page in allocator
  int size() ;                 // debug method
  int isused(PID, int len) ;   // debug method
} ;


inline void Volume::sync() {
  SYSCALLI(fsync(this->_fd)) ;
}

inline int Volume::volid() {
  return _volid ;
}

inline Volume::Volume(char* name,int id) : _volname(strdup(name)), _volid(id) {
}

inline int Volume::size() {
  return _size ;
}

#endif
